/*
 * Copyright (c) 2014, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * */

// use jsdoc ds.js to generate documentation
/**
 * Debug Server
 * Main access point for utilizing debug server API's.
 * @namespace
 */
var DS = DS || {};

// base error
DS.Error = function (name, msg) {
    if (name)
        this.name = name;
    else
        this.name = "DS.Error";

    if (msg)
        this.msg = msg;
    else
        this.msg = "ERROR";
};
DS.Error.prototype = new Error("DS.Error", "ERROR");

// Tools version mismatch
DS.InvalidVersion = function () {
};
DS.InvalidVersion.prototype = new DS.Error("DS.InvalidVersion", "INVALID_VERSION");

// missing paramter
DS.MissingRequiredParameter = function (param) {
    this.param = param;
};
DS.MissingRequiredParameter.prototype = new DS.Error("DS.MissingRequiredParameter", "MISSING_REQUIRED_PARAMETER");

DS.ConnectionToTICloudAgentFailed = function () {
};
DS.ConnectionToTICloudAgentFailed.prototype = new DS.Error("DS.ConnectionToTICloudAgentFailed", "Socket Connection Error");

// socket cannot be open
DS.MissingSSLCertificate = function () {
};
DS.MissingSSLCertificate.prototype = new DS.Error("DS.MissingSSLCertificate", "MISSING_SSL_CERTIFICATE");

// OS Enum
DS.OS = {
    WIN : "win",
    LINUX : "linux",
    LINUX64 : "linux64",
    OSX : "osx"
};

// utility function to figure out OS
var getOS = function() {
    var os = DS.OS.WIN;
    if (navigator.appVersion.indexOf("Mac")!=-1) os=DS.OS.OSX;
    if (navigator.appVersion.indexOf("Linux")!=-1) os=DS.OS.LINUX;
    if (typeof process !== 'undefined'){
        var execPath = process.execPath;
        if (execPath !== undefined){
            if (execPath.toLowerCase().indexOf("linux64") >= 0){
                os = DS.OS.LINUX64;
            }
        }
    }
    console.log("ccs_debug.getOS: appVersion = "+navigator.appVersion);
    console.log("ccs_debug.getOS: platform="+navigator.platform);
    console.log("ccs_debug.getOS: userAgent="+navigator.userAgent);
    console.log("ccs_debug.getOS: returning "+os);
    return os;
};

// if we are in the node-webkit context define a StartCloudAgent function
// else use the browser specific code is needed. 
if( typeof process != 'undefined' ) {

	 /**
	 * Start cloud agent parameters within node webkit
	 * @typedef {Object} startCloudAgentNodeWebkitParams
	 * @property {string} appRoot root path of the node webkit application. The cloud agent path is calculated from here. Assume node webkit if this is set.
	 */

	/**
     * Start cloud agent within node webkit context
     * @param {startCloudAgentNodeWebkitParams} startCloudAgentNodeWebkitParams
     * @param {apiCallCompleteCallBack} complete called when the start process is complete
     * @memberof DS
     * @function DS.StartCloudAgentNodeWebkit
     */
	DS.StartCloudAgentNodeWebkit = function( startCloudAgentNodeWebkitParams , complete ) {		

		var exec = window.require('child_process').execFile;
		var path = window.require( 'path' );
		var http = window.require("http");
		var url = window.require("url");
		
		var extension = getOS() === DS.OS.WIN ? ".exe" : "";

		var osSpecificFolder = "win32";
		switch(getOS()){
		  case DS.OS.OSX: osSpecificFolder = "mac32";break;
		  case DS.OS.LINUX: osSpecificFolder = "linux32";break;
		  case DS.OS.LINUX64: osSpecificFolder = "linux64";break;
		  case DS.OS.WIN: osSpecificFolder = "win32";break;
		}
		var appRootPath =  startCloudAgentNodeWebkitParams.appRoot;
		var cloudAgentPath = path.join(appRootPath ,osSpecificFolder);
		console.log("escaped cloudAgentPath = "+cloudAgentPath);
		// Need to be able to support file folder names that use unicode characters (e.g. Kanji)
		// as well as folder names with spaces in them.  In both cases, the folder names will be
		// passed in using %<hex character> escape notation.  Convert back to unicode for use
		// when starting the TICloudAgent.
		cloudAgentPath = unescapeUnicode(cloudAgentPath);

		cloudAgentPath = path.join( cloudAgentPath , "TICloudAgent" );
		var nodeExe = path.join(cloudAgentPath,"node" + extension);
			
		// set up a temp http server to listen for cloud agent registration
		var server = http.createServer( function(request, response) {
				
			var urlParts = url.parse( request.url,true );
			var uri = urlParts.pathname;
			var data =  urlParts.query;
			console.log( "Cloud agent registration complete: " + data );
			// respond okay
			response.writeHead(200);
			response.end();
			
			server.close(); // close the server
			complete( data ); // start complete
				
		});	
		server.listen();
		
		// start cloud agent
		var httpPort = server.address().port;
		console.log( "Waiting for agent registration on port " + httpPort);

		var regAPI = "http://localhost:" + httpPort + "/DS.StartCloudAgentNodeWebkit";
		var agentArgs = "ticloudagent://?module=debug&sessionID=none&registrationAPI=" + regAPI;
		

		var localJsFilePath = path.join(cloudAgentPath,"src/local.js");
		console.log("StartCloudAgentNodeWebkit: node "+localJsFilePath + " " + agentArgs);
		var args = [ localJsFilePath , agentArgs];

        var fs = require('fs');
        fs.chmodSync(nodeExe, '755');

		exec( nodeExe , args, { cwd: cloudAgentPath }, function (err, data) {
			console.log( "Error : " + err );
			console.log( "data : " + data );
		});
	};	

}


function unescapeUnicode(str){
	var result = str.replace(/((%[0-9a-fA-F]{2})+)/g, 
		function(value,strToConvert) { 
			return getUnicodeStr(strToConvert);
		});
	return result;

}

function getUnicodeStr(str){ 

	var result = "";

	var hexCodesAsStrings = str.split('%');
	var hexCodes = [];
	for ( var j = 1; j < hexCodesAsStrings.length; j++){
		hexCodes[j-1] = parseInt(hexCodesAsStrings[j],16);
	}
	var charCode = 0;
	var charCode1 = 0;
	var charCode2 = 0;
	var charCode3 = 0;
	var charCode4 = 0;
	var hexCode = 0;
	var page = 0;
	var i = 0;
	var OK = true;

	while ((i < hexCodes.length)&&(OK)) {
		hexCode = hexCodes[i++];

		if ((hexCode >= 0) && (hexCode <= 0x7F)){
			charCode = hexCode;
			result = result + String.fromCharCode(charCode);
		} else {
			if ((i >= hexCodes.length) ||
			   (hexCodes[i] <= 0x80) || (hexCodes[i] >= 0xBF) ) {
			   OK = false;
		    } else {
			if ((hexCode >= 0xC0) && (hexCode <= 0xDF)) {   
				charCode2 = (hexCode & 0x1F);
				charCode1 = hexCodes[i++] - 0x80;
				charCode = (charCode2 << 6) | charCode1;
			    result = result + String.fromCharCode(charCode);
			} else {
				if ((i >= (hexCodes.length - 1)) ||
					(hexCodes[i]   <= 0x80) || (hexCodes[i]   >= 0xBF) ||
					(hexCodes[i+1] <= 0x80) || (hexCodes[i+1] >= 0xBF) ) {

			        OK = false;
			    } else {
			        if ((hexCode >= 0xE0) && (hexCode <= 0xEF)) {   
					charCode3 = (hexCode & 0xF);
					charCode2 = hexCodes[i++] - 0x80;
					charCode1 = hexCodes[i++] - 0x80;
					charCode = (((charCode3 << 6) | charCode2) << 6) | charCode1;
    			    result = result + String.fromCharCode(charCode);
	
					} else {
						if ((i >= (hexCodes.length - 2)) ||
							(hexCodes[i]   <= 0x80) || (hexCodes[i]   >= 0xBF) || 
							(hexCodes[i+1] <= 0x80) || (hexCodes[i+1] >= 0xBF) ||
							(hexCodes[i+2] <= 0x80) || (hexCodes[i+2] >= 0xBF) ) {
							OK = false;
						} else {
							if ((hexCode >= 0xF0)&&(hexCode <= 0xF7)){
								charCode4 = (hexCode & 0x7);
								charCode3 = hexCodes[i++] - 0x80;
								charCode2 = hexCodes[i++] - 0x80;						
								charCode1 = hexCodes[i++] - 0x80;
								charCode = ((((charCode4 << 6) | charCode3 << 6) | charCode2) << 6) | charCode1;
								result = result + String.fromCharCode(charCode);
								
							} else {

								OK = false;
							}
						}
					}
	           }
			}
	      }
		} 

	}
    if (!OK) {
		console.log("ti-core-backplane: ccs_debug failed to convert escaped path to unicode at index = "+i);
	}
	return (result);
}

// socket cannot be open
DS.Timeout = function () {
};
DS.Timeout.prototype = new DS.Error("DS.Timeout", "TIMEOUT");

DS.Init = function () {

    var VERSION = ".2";
    var OS;

    // MSG_TYPES
    var MSG_TYPES = {
        WAITING_FOR_BINARY_DATA: "DS_WAITING_FOR_BINARY_DATA",
        ERROR: "DS_ERROR",
        SUCCESS: "DS_SUCCESS",
        OUTPUT_TEXT: "DS_OUTPUT_TEXT",
        OUTPUT_SERIAL_TEXT: "DS_SERIAL_TEXT"
    };

    this.foundFlashMsg = false;
    this.timeoutSet = false;

    var _self = this;
    // bc: moved waitForSccketConnection here so that it can be used generically
    var waitForSocketConnection = function (socket, callback) {
        setTimeout(
            function () {
                if (socket.readyState === 1) {
                    if (callback !== undefined) {
                        callback();
                    }
                    return;
                } else {
                    // keep waiting if socket is connecting
                    if (socket.readyState == 0) {
                        waitForSocketConnection(socket, callback);
                    } else {
                        //TODO: handle case where socket is closing (2) or closed(3)
                        console.log("ERROR: webSocket readyState =" + socket.readyState + ', (2=closing, 3=closed)');
                    }
                }
            }, 5);
    };
    var createBaseMessageHandler = function (output, callback, input) {

        var baseMessageHandler = {};
        baseMessageHandler[MSG_TYPES.WAITING_FOR_BINARY_DATA] = function (message) {
        };
        baseMessageHandler[MSG_TYPES.ERROR] = function (message) {
            if ((output !== undefined) && (output !== null)) {
                output.text(message.data);
                callback( new DS.Error() );
            }
            console.log('ccs_debug baseMessageHandler: error = ' + message.data);
            callback(new DS.Error("error", message.data), message.data);
        };
        baseMessageHandler[MSG_TYPES.SUCCESS] = function (message) {
            if (((typeof message.data) === 'string') && (output !== undefined) && (output !== null)) {
                output.text(message.data);
                console.log('ccs_debug baseMessageHandler: success = ' + message.data);
                if (message.data.toLowerCase().indexOf('flash successful') >= 0) {
                    console.log('ccs_debug baseMessageHandler: setting foundFlashMsg = true');
                    _self.foundFlashMsg = true;
                    //window.clearTimeout();
                    _self.timeoutSet = false;
                }
            } else {
                console.log('ccs_debug baseMessageHandler: success! about to call callback (e.g. programLoadComplete)');
            }

            callback(null, input ? input : message.data);
        };
        baseMessageHandler[MSG_TYPES.OUTPUT_TEXT] = function (message) {
            if ((message === null) || (message === undefined) || (message.data === undefined) || (message.data === null)) {
                console.log('ccs_debug baseMessageHandler: output_text - null message!');

            } else {
                console.log('ccs_debug baseMessageHandler: output_text = ' + message.data);
                // possible messages: Flashing Device... Flash Successful!
                if (message.data.toLowerCase().indexOf('flash successful') >= 0) {
                    console.log('ccs_debug baseMessageHandler: setting foundFlashMsg = true');
                    _self.foundFlashMsg = true;
                    //window.clearTimeout();
                    _self.timeoutSet = false;
                }
            }
            if ((output !== undefined) && (output !== null)) {
                output.text(message.data);
            }
        };
        baseMessageHandler.handle = function (msgEvt) {
            var message = JSON.parse(msgEvt.data);
            var handler = this[message.msgType];
            handler ? handler(message) : console.log("Unhandled Message Type : " + message.msgType);
        };

        return baseMessageHandler;
    };

    var requestID = 0;

    var socketUrl = function (port, isSecureWebPage) {
            if (isSecureWebPage) {
                return "wss://localhost:" + port + "/socket/";
            } else {
                return "ws://localhost:" + port + "/socket/";
            }
    };

    // documentation for custom types

    /**
     * All debug server API's accept an object of this type. Used by the API's to send info/error/warning messages.
     *
     * @typedef {Object} output
     * @property {function} text handle text output
     */

    /**
     * Some debug server API's return an object of this type to accept user input
     *
     * @typedef {Object} input
     * @property {function} text handle text input
     * @property {function} close handle close
     */

    /**
     * Generic call back interface for returning results of a ds API call which supports user input
     *
     * @callback apiCallCompleteInputCallBack
     * @param {object} error null if successful, DS.Error otherwise
     * @param {input} input object
     *
     */

    /**
     * Generic call back interface for returning results of a ds API call
     *
     * @callback apiCallCompleteCallBack
     * @param {object} error null if successful, DS.Error otherwise
     * @param {object} result result of the api call [optional]
     *
     */

    /**
     * DS.Init callback for report success/failure and a debug server {ds} object which could be used
     * to access the available API's.
     *
     * @callback initCompleteCallBack
     * @param {object} error null if successful, DS.Error otherwise, DS.InvalidVersion if version mistmatch
     * @param {ds} ds debug server object
     */

    /**
     * Initialize DS
     * @param port Local server port
     * @param {initCompleteCallBack} initComplete Called when initialization is complete
     * @memberof DS
     * @function DS.Init
     */

    return function (initParams, initComplete, initFailedCallback, terminateCallback, retryCallback, callerObj) {

        if (initParams.version !== VERSION) {
            initComplete(new DS.InvalidVersion());
            return (false);
        }
		if( !initParams.port ) {
			//use the default port
			console.log( "No agent port provided. Default to " + TI_CLOUD_AGENT_WS_PORT );
			initParams.port = TI_CLOUD_AGENT_WS_PORT;
		}

        // set up the main loop websocket
        callerObj.mainSocket = new WebSocket(socketUrl(initParams.port, initParams.isSecureWebPage));
        var socketHasBeenOpened = false;
        var mainSocketError = false;

        callerObj.mainSocket.onclose = function () {
            if (( !socketHasBeenOpened) && (!mainSocketError)) {
                if (initComplete) {
                    initComplete(new DS.MissingSSLCertificate());
                }
                return (false);
            }
            if (terminateCallback) {
                terminateCallback();
            }
        };

        callerObj.mainSocket.onerror = function () {
            console.log("main socket error. About to call initFailedCallback with error = ConnectionToTICloudAgentFailed...");
            mainSocketError = true;
            if (retryCallback) {
                retryCallback();
            } else {
                if (initFailedCallback) {
                    initFailedCallback(new DS.ConnectionToTICloudAgentFailed());
                }
            }
        };

        callerObj.mainSocket.onopen = function () {
            socketHasBeenOpened = true;
            var socketCache = {};

            // create a socket for a command
            // a cache is kept where the target + command are used as the key
            var createSocket = function (target, commandData, createComplete) {

                var key = target + "_" + commandData.command;
                var socket = socketCache[key];
                if (!socket) {
                    callerObj.mainSocket.send(JSON.stringify(commandData));
                    callerObj.mainSocket.onmessage = function (msgEvt) {

                        var data = JSON.parse(msgEvt.data);
                        // create a local socket to facilitate load program
                        socket = new WebSocket(socketUrl(data.port, initParams.isSecureWebPage));
                        socketCache[key] = socket;
                        socket.onopen = function () {
                            if (createComplete) {
                                createComplete(socket);
                            }
                        };

                        socket.onclose = function () {
                            delete socketCache[key];// remove ourselves from the cache
                        };

                    };
                } else {
                    // send socket indicating it was cached
                    if (createComplete) {
                        createComplete(socket, true);
                    }
                }
            };


            var debug = function () {

                var targetDebugCache = {};

                return function (target) {

                    var targetDebug = targetDebugCache[target];

                    if (!targetDebug) {

                        /**
                         * Provides access to target specific debug API's
                         * @namespace targetDebug
                         */
                        targetDebug = {

                            /**
                             * Load program parameters
                             * @typedef {Object} loadProgramParams
                             * @property {Object} data binary data to be written
                             * @property {number} loadAddress load address [optional]
                             * @property {number} runAddress run address [optional]
                             */

                            /**
                             * Load a program
                             * @param ouptut {output} output object
                             * @param {loadProgramParams} loadParams
                             * @param {apiCallCompleteCallBack} callback
                             * @memberof targetDebug
                             * @function
                             */
                            loadProgram: function () {

                                var validateParams = function (loadParams, programLoadComplete) {
                                    console.log('ccs_debug.debug.loadProgram: loadParams.length=' + loadParams.length);
                                    if (( target !== undefined) && (target.indexOf('TMS320F28') >= 0 && !loadParams.comPort )) {
                                        programLoadComplete(new DS.MissingRequiredParameter("comPort"));
                                        return false;

                                    }

                                    return true;
                                };
                                _self.foundFlashMsg = false;

                                return function (output, loadParams, programLoadComplete) {
                                    console.log('in ccs_debug.debug.loadProgram return function: loadParams.name=' + loadParams.name);
                                    if (validateParams(loadParams, programLoadComplete)) {

                                        createSocket(target, {command: "debug.loadProgram"}, function (loadProgramSocket) {

                                            console.log('ccs_debug.debug.loadProgram.createSocket return function: loadProgramSocket=' + loadProgramSocket);
                                            // bc: moved waitForSocketConnection to top level so that it can be used generically
                                            /*
                                             var waitForSocketConnection = function (socket, callback){
                                             setTimeout(
                                             function(){
                                             if (socket.readyState === 1) {
                                             if(callback !== undefined){
                                             callback();
                                             }
                                             return;
                                             } else {
                                             // keep waiting if socket is connecting
                                             if (socket.readyState == 0){
                                             waitForSocketConnection(socket,callback);
                                             } else {
                                             //TODO: handle case where socket is closing (2) or closed(3)
                                             console.log("ERROR: webSocket readyState ="+socket.readyState+', (2=closing, 3=closed)');
                                             }
                                             }
                                             }, 5);
                                             };
                                             */
                                            var sendProgramLoadContext = function () {

                                                var programLoadContext = {
                                                    requestID: requestID++,
                                                    target: target,
                                                    file: loadParams.name,
                                                    comPort: loadParams.comPort ? loadParams.comPort : ""
                                                };
                                                console.log('ccs_debug.debug.loadProgram.createSocket return function: Starting to wait for socket...');
                                                //bc: added the following since it sometimes would error out with 'Failed to execute 'send' on 'WebSocket': Still in CONNECTING state.
                                                waitForSocketConnection(loadProgramSocket, function () {
                                                    console.log('ccs_debug.debug.loadProgram.createSocket return function: Socket ready.  programLoadContext.file=' + programLoadContext.file + ': Starting to wait for socket...');
                                                    loadProgramSocket.send(JSON.stringify(programLoadContext));
                                                });

                                            };

                                            var sendProgramData = function () {
                                                console.log('sendProgramData: loadParams.data.byteLength = ' + loadParams.data.byteLength);
                                                loadProgramSocket.send(loadParams.data);
                                            };

                                            loadProgramSocket.onmessage = function (msgEvnt) {
                                                if ((msgEvnt !== undefined) && (msgEvnt.data !== undefined)) {
                                                    console.log('ccs_debug.debug.loadProgramSocket.onmessage msgEvnt=' + msgEvnt.data);
                                                }

                                                var handler = createBaseMessageHandler(output, programLoadComplete);
                                                handler[MSG_TYPES.WAITING_FOR_BINARY_DATA] = function (message) {
                                                    console.log('ccs_debug.debug.loadProgramSocket.onmessage about to sendProgramData...');
                                                    sendProgramData();
                                                };
                                                handler.handle(msgEvnt);
                                            };

                                            waitForSocketConnection(loadProgramSocket, function () {
                                                sendProgramLoadContext(); // start stage one
                                            });

                                        });
                                    }
                                };
                            }()
                        };
                        targetDebugCache[target] = targetDebug;
                    }
                    ;

                    return targetDebug;
                };
            }();

            /**
             * Serial port API's
             *
             * @namespace serial
             */
            var serial = {

                /**
                 * Open serial port parameters
                 * @typedef {Object} openSerialParams
                 * @property {string} comName name of the port to open
                 * @property {string} baudrate baudrate
                 */

                /**
                 * Open a connection to a serial port
                 * @param {output} output output object
                 * @param {output} serialOutput serial port read output object. Only messages from the serial port are re-directed here.
                 * @param {openSerialParams} openParams
                 * @param {apiCallCompleteInputCallBack} callback
                 * @memberof serial
                 * @function
                 */
                open: function (output, serialOutput, openParams, openComplete) {

                    // decorate the paramaters to pass along
                    openParams.command = "serial.open";
                    this.serialOutput = serialOutput;  //bc NEW - persist callback function handle
                    var dataReceived = false;
                    var _self = this;
                    createSocket(openParams.comName, openParams, function (serialPortSocket, cached) {
                        if (cached) {
                            if ((output !== undefined) && (output !== null)) {
                                output.text(openParams.comName + " is already open");
                                openComplete( new DS.Error() );
                                return;
                            }

                            openComplete(new DS.Error("createSerialPortSocketFailed", openParams.comName + " is already open"), openParams.comName + " is already open");

                            return;
                        }

                        var closeSerialPort = function () {
                            if ((output !== undefined) && (output !== null)) {
                                output.text("Closing " + openParams.comName);
                            }
                            _self.dataReceived = false;
                            //window.clearTimeout();
                            serialPortSocket.close();
                        };

                        var input = {
                            text: function (data) {
                                _self.dataReceived = true;
                                //window.clearTimeout();
                                serialPortSocket.send(data);
                            }
                        };

                        serialPortSocket.onmessage = function (msgEvnt) {

                            var handler = createBaseMessageHandler(output, openComplete, input);

                            handler[MSG_TYPES.ERROR] = function (message) {
                                if (output !== undefined) {
                                    output.text(message.data);
                                }
                                closeSerialPort();
                                openComplete(false);
                            };

                            handler[MSG_TYPES.OUTPUT_SERIAL_TEXT] = function (message) {
                                if (_self.serialOutput !== undefined) {
                                    _self.dataReceived = true;
                                    _self.serialOutput.text(message.data);
                                }
                            };

                            handler.handle(msgEvnt);
                        };

                        // object to accept input
                        var input = {
                            text: function (data) {
                                serialPortSocket.send(JSON.stringify(data));
                            },
                            close: closeSerialPort
                        };
                    });
                },
                /**
                 * get a list of serial ports
                 * @param {output} output object
                 * @param {loadProgramParams} openParams
                 * @param {apiCallCompleteCallBack} listComplete
                 * @memberof serial
                 * @function
                 */
                list: function () {

        /*            var keywords = ["Texas Instruments", "MSP430", "FTDI"];  */

                    var setUpDisplayName = function (ports) {
                        for (var i = 0; i < ports.length; ++i) {
                            var port = ports[i];
                            if (port.manufacturer) {
                                port.displayName = port.comName + "(" + port.manufacturer + ")";
                            } else if (port.pnpId) {
                                port.displayName = port.comName + "(" + port.pnpId + ")";
                            } else {
                                port.displayName = port.comName;
                            }
                        }

                    };
/*
                    var selectDefault = function (ports, target) {

                        var found = false;
                        var numFound = 0; // bc Hack
                        for (var i = 0; i < ports.length; ++i) {
                            for (var j = 0; j < keywords.length; ++j) {
                                if (ports[i].displayName.indexOf(keywords[j]) != -1) {
                                    ports[i].selected = true;
                                    found = true;
                                    //bc Hack [...
                                    numFound++;
                                    //break;
                                    //...]
                                }
                            }
                            // bc Hack: [...
                            // if( found )
                            // break;
                             // ...] 
                        }

                        //bc Hack for F5529, which has 2 serial ports.
                        // The Application serial port has pnpID = 0002
                        // on windows, or 0200 on Mac, so
                        // use that to determine which port to use
                        // [...
                        var isMac = false;
                        if (navigator.appVersion.indexOf("Mac") != -1) {
                            isMac = true;
                        }
                        ;

                        if (((numFound > 1) || (isMac && (target.indexOf('F5529') > -1)))) {
                            var numChecked = 0;
                            var isClearSelected = false;
                            var pnpId = "0002";
                            if (isMac) {
                                pnpId = "0200";
                            }

                            // Select the port that has the keyword 'Application' in it.
                            // If neither do, select the last one.
                            for (var i = 0; i < ports.length; ++i) {
                                if ((ports[i].selected) || (isMac)) {
                                    if (ports[i].pnpId.indexOf(pnpId) > -1) {
                                        ports[i].selected = true;
                                        console.log('ccs_debug: selecting port[' + i + '].pnpId = ' + ports[i].pnpId);
                                    } else {
                                        ports[i].selected = false;
                                        console.log('ccs_debug: de-selecting port[' + i + '].pnpId = ' + ports[i].pnpId);
                                    }
                                } else {
                                    ports[i].selected = false;
                                }
                            }
                        }
                        //...]
                        if (!found && ports.length > 0) {
                            ports[0].selected;
                        }

                    };
*/
                    return function (output, listComplete) {

                        var commandData = {command: "serial.list"};

                        createSocket("", commandData, function (listSerialSocket, cached) {

                            listSerialSocket.onmessage = function (msgEvnt) {

                                var handler = createBaseMessageHandler(output, listComplete);
                                ;

                                // override the SUCCESS callback to select default port
                                handler[MSG_TYPES.SUCCESS] = function (message) {
                                    setUpDisplayName(message.data);
                                   /* selectDefault(message.data);  */
				    selectDefaultPort(output,message.data,listComplete);
                                    listComplete(null, message.data);
                                };

                                handler.handle(msgEvnt);
                            };

                            if (cached) {
                                listSerialSocket.send(JSON.stringify(commandData));
                            }

                        });
                    };

                }(),

                selectDefaultPort :

                /**
                 * Load program parameters
                 * @typedef {Object} selectDefaultPortParams
                 * @property {array} ports ports to select from
                 * @property {String} target target variant
                 */

                /**
                 * select the default port
                 * @param {output} output object
                 * @param {selectDefaultPortParams} params
                 * @param {apiCallCompleteCallBack} selectDefaultPortComplete
                 * @memberof serial
                 * @function
                 */
                    function() {

                        var keywords = ["Texas Instruments", "MSP", "FTDI"];

                        // iterate over all ports with matching key word and perform additional action to determine
                        // if it is the port that needs to be selected
                        var iteratePortsWithMatchingKeyWords = function( ports, action ) {

                            var found = false;
                            for( var i = 0; i < ports.length; ++i) {
                                for( var j = 0; j < keywords.length; ++j) {
                                    if( ports[i].displayName.indexOf(keywords[j]) != -1) {
                                        found = action( ports[i] );
                                        ports[i].selected = found;
                                        if( found )
                                            break;
                                    }
                                }
                                if( found )
                                    break;
                            }

                            if( !found && ports.length > 0 )
                                ports[0].selected = true;
                        };

                        // default is to select any port matching the keyword
                        var selectDefault = function( ports ) {
                            iteratePortsWithMatchingKeyWords( ports, function( port ){
                                return true;
                            } );
                        };

                        // this is the default selection for 5529 and 5969 msp430's on linux/windows
                        // the pnpid which refers to the interface should always be 02 on linux/windows
                        // as per MSP team instructions
                        var selectDefaultMSP5529And5969 = function ( ports ) {
                            iteratePortsWithMatchingKeyWords( ports, function( port ){
                                var suffix = "02";
                                 return port.pnpId.indexOf(suffix, this.length - suffix.length) !== -1;
                            } );
                        };

                        // this is the default selection for 5529 and 5969 msp430's
                        // on osx; which is an special case
                        var selectDefaultMSP5529And5969OnOsx = function ( ports ) {
                            var found = false;
                            for( var i = 0; i < ports.length; ++i) {
                                if( ports[i].comName.indexOf("/dev/cu.usb") !== -1 &&  ports[i].manufacturer === "" ) {
                                    found = true;
                                    ports[i].selected = found;
                                }

                                if( found )
                                    break;
                            }

                            if( !found && ports.length > 0 )
                                ports[0].selected = true;
                        };

                        return function ( output, selectParams, selectDefaultPortComplete) {

                            var ports = selectParams.ports;

                            var selectFunc = selectDefault;
                            // msp430 5529 and 5969
                            if( selectParams.target.indexOf("5529") > -1 ||  selectParams.target.indexOf("5969") > -1 ) {
                                if( getOS() === DS.OS.OSX )
                                    selectFunc = selectDefaultMSP5529And5969OnOsx;
                                else
                                    selectFunc = selectDefaultMSP5529And5969;
                            }

                            selectFunc( ports );
                          //  selectDefaultPortComplete();
                            selectDefaultPortComplete(null,selectParams);

                        };
                }(),

                /**
                 *
                 * @param {output} output object
                 * @param {apiCallCompleteCallBack} baudRatesComplete
                 */

                baudRates: function () {

                    var baudRates = [
                        {rate: "50", selected: false},
                        {rate: "75", selected: false},
                        {rate: "110", selected: false},
                        {rate: "134", selected: false},
                        {rate: "150", selected: false},
                        {rate: "300", selected: false},
                        {rate: "600", selected: false},
                        {rate: "1200", selected: false},
                        {rate: "1800", selected: false},
                        {rate: "2400", selected: false},
                        {rate: "4800", selected: false},
                        {rate: "7200", selected: false},
                        {rate: "9600", selected: true},
                        {rate: "14400", selected: false},
                        {rate: "19200", selected: false},
                        {rate: "28800", selected: false},
                        {rate: "38400", selected: false},
                        {rate: "56000", selected: false},
                        {rate: "57600", selected: false},
                        {rate: "115200", selected: false},
                        {rate: "128000", selected: false},
                        {rate: "153600", selected: false},
                        {rate: "230400", selected: false},
                        {rate: "256000", selected: false},
                        {rate: "460800", selected: false},
                        {rate: "921600", selected: false}];

                    return function (output, baudRatesComplete) {
                        baudRatesComplete(null, baudRates);
                    };
                }()
            };


            /**
             * The main object which exposes the debug server API's
             * @namespace ds
             */
            var ds = {

                /**
                 * Function object which provides access to a target debug object
                 * @param target parameter describing the target
                 * @returns {targetDebug} target debug object
                 * @memberof ds
                 * @function
                 */
                debug: debug,

                /**
                 * Object providing access to serial port API's
                 * @type {serial}
                 * @memberof ds
                 * @instance
                 */
                serial: serial
            };

            // bc: ensure socket is ready before proceeding to initComplete
            waitForSocketConnection(callerObj.mainSocket, function () {
                initComplete(null, ds);
            });
            return (true);
        };
    };
}();

/* bc: remove stuff related specifically to cloud9
 module.exports = m_ext.register("ext/ccs_debug/ccs_debug", {

 // this is mandatory
 type	: m_ext.GENERAL,

 // these seem optinal but it is a good idea to have them.
 name	: "TI Cloud Agent Extensions",
 dev		: "Texas Instruments",
 //		alone	: true,
 //		deps	: [],
 //		enable	: function() {},
 //		disable	: function() {},
 //		destroy	: function() {},
 //		init	: function() {},

 // custom logic.
 DS   : DS,
 });


 });
 */
